var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Once } from 'lodash-decorators';
var Listener = (function () {
    function Listener(store) {
        this.store = store;
        this.listeners = {
            mutationListeners: {},
            beforeActionListeners: {},
            afterActionListeners: {}
        };
        this.addMutationListener = this.createSubscriber('mutationListeners', this.subscribeToMutations.bind(this));
        this.addBeforeActionListener = this.createSubscriber('beforeActionListeners', this.subscribeToActions.bind(this));
        this.addAfterActionListener = this.createSubscriber('afterActionListeners', this.subscribeToActions.bind(this));
    }
    Listener.prototype.subscribeToMutations = function () {
        var _this = this;
        this.store.subscribe(function (mutation, state) {
            if (_this.listeners.mutationListeners[mutation.type] === undefined)
                return;
            _this.listeners.mutationListeners[mutation.type].forEach(function (listener) { return listener(mutation.payload, state); });
        });
    };
    Listener.prototype.subscribeToActions = function () {
        var _this = this;
        this.store.subscribeAction({
            before: function (action, state) {
                if (_this.listeners.beforeActionListeners[action.type] === undefined)
                    return;
                _this.listeners.beforeActionListeners[action.type].forEach(function (listener) { return listener(action.payload, state); });
            },
            after: function (action, state) {
                if (_this.listeners.afterActionListeners[action.type] === undefined)
                    return;
                _this.listeners.afterActionListeners[action.type].forEach(function (listener) { return listener(action.payload, state); });
            }
        });
    };
    Listener.prototype.createSubscriber = function (listeners, subscribe) {
        var _this = this;
        return function (key, listener) {
            subscribe();
            if (_this.listeners[listeners][key] === undefined) {
                _this.listeners[listeners][key] = [];
            }
            _this.listeners[listeners][key].push(listener);
            return function () {
                _this.listeners[listeners][key] = _this.listeners[listeners][key].filter(function (l) { return l !== listener; });
            };
        };
    };
    __decorate([
        Once
    ], Listener.prototype, "subscribeToMutations", null);
    __decorate([
        Once
    ], Listener.prototype, "subscribeToActions", null);
    return Listener;
}());
export { Listener };
export var VuexListener = {
    install: function (Vue) {
        Vue.mixin({
            beforeCreate: function () {
                var options = this.$options;
                if (options.store) {
                    this.$storeListener = new Listener(options.store);
                }
                else if (options.parent && options.parent.$storeListener) {
                    this.$storeListener = options.parent.$storeListener;
                }
            }
        });
    }
};
